document.addEventListener('DOMContentLoaded', function() {
    // 1. 首先初始化所有变量
    const tpsHistory = Array(30).fill(20);
    const msptHistory = Array(30).fill(50);
    let memoryChart = null;
    let performanceChart = null;
    let lang = {}; // 新增：存储语言文件内容

    // 2. 获取DOM元素（带安全检查）
    const elements = {
        refreshBtn: document.getElementById('refresh-btn'),
        themeToggle: document.getElementById('theme-toggle'),
        statsTableBody: document.getElementById('stats-table')?.getElementsByTagName('tbody')[0], // Changed ID for clarity
        statsTableHeader: document.getElementById('stats-table')?.getElementsByTagName('thead')[0], // Added for header localization
        whitelistPlayers: document.getElementById('whitelist-players'),
        nonWhitelistPlayers: document.getElementById('non-whitelist-players'),
        totalCount: document.getElementById('total-count'),
        whitelistCount: document.getElementById('whitelist-count'),
        nonWhitelistCount: document.getElementById('non-whitelist-count'),
        memoryUsed: document.getElementById('memory-used'),
        memoryFree: document.getElementById('memory-free'),
        memoryPercent: document.getElementById('memory-percent'),
        avgTick: document.getElementById('avg-tick'), // This element seems unused in updateServerStatus now, can remove if not needed
        uptime: document.getElementById('uptime'),
        tpsValue: document.getElementById('tps-value'),
        msptValue: document.getElementById('mspt-value')
    };

    // 3. 初始化主题
    const savedTheme = localStorage.getItem('theme') || 'light';
    document.documentElement.setAttribute('data-theme', savedTheme);

    // 4. 初始加载语言文件，然后加载其他数据并初始化图表
    loadLanguage().then(() => {
        translatePage(); // 本地化页面静态文本
        initCharts(); // 初始化图表（需要语言文本）
        loadAllData(); // 加载动态数据
        // 7. 设置定时刷新（10秒）
        const refreshInterval = setInterval(loadAllData, 10000);
    }).catch(error => {
        console.error('Failed to load language or initial data:', error);
        showError('Failed to load language or initial data.'); // Fallback error message
    });


    // 5. 事件监听器
    if (elements.themeToggle) {
        elements.themeToggle.addEventListener('click', toggleTheme);
    }

    if (elements.refreshBtn) {
        elements.refreshBtn.addEventListener('click', handleRefresh);
    }

    /*** 功能函数 ***/

    // 新增：加载语言文件
    async function loadLanguage() {
        try {
            const response = await fetch('/api/lang');
            if (!response.ok) throw new Error(`HTTP error! status: ${response.status}`);
            lang = await response.json();
            console.log('Language file loaded.');
        } catch (error) {
            console.error('Failed to load language file:', error);
            // Fallback to English keys if language file fails to load
            lang = {};
            throw error; // Propagate error to stop further loading if language is critical
        }
    }

    // 新增：根据data-lang-key属性本地化页面元素
    function translatePage() {
        document.querySelectorAll('[data-lang-key]').forEach(element => {
            const key = element.getAttribute('data-lang-key');
            if (lang[key]) {
                // Special handling for title tag
                if (element.tagName === 'TITLE') {
                    document.title = lang[key];
                } else {
                    element.textContent = lang[key];
                }
            } else {
                console.warn(`Missing translation key: ${key}`);
            }
        });
    }

    // 新增：获取本地化字符串
    function getLangString(key, ...args) {
        const pattern = lang[key] || key; // Use key as fallback
        try {
            // Simple placeholder replacement {0}, {1}, etc.
            let result = pattern;
            args.forEach((arg, index) => {
                result = result.replace(new RegExp('\\{' + index + '\\}', 'g'), arg);
            });
            return result;
        } catch (e) {
            console.error(`Failed to format string for key: ${key}`, e);
            return pattern; // Return unformatted pattern on error
        }
    }


    function initCharts() {
        // 内存图表
        const memoryCtx = document.getElementById('memory-chart')?.getContext('2d');
        if (memoryCtx) {
            memoryChart = new Chart(memoryCtx, {
                type: 'doughnut',
                data: {
                    // Use localized labels
                    labels: [getLangString('playertime.web.chart.memory_used'), getLangString('playertime.web.chart.memory_free')],
                    datasets: [{
                        data: [0, 100],
                        backgroundColor: ['#4361ee', '#e9ecef']
                    }]
                },
                options: {
                    responsive: true,
                    maintainAspectRatio: false,
                    plugins: {
                        legend: {
                            position: 'top',
                        }
                    }
                }
            });
        }

        // 性能图表
        const perfCtx = document.getElementById('performance-chart')?.getContext('2d');
        if (perfCtx) {
            performanceChart = new Chart(perfCtx, {
                type: 'line',
                data: {
                    labels: Array(30).fill(''),
                    datasets: [
                        {
                            // Use localized labels
                            label: getLangString('playertime.web.chart.tps'),
                            data: tpsHistory,
                            borderColor: '#4CAF50',
                            backgroundColor: 'rgba(76, 175, 80, 0.1)',
                            borderWidth: 2,
                            pointRadius: 3,
                            pointHoverRadius: 5,
                            tension: 0.3,
                            yAxisID: 'y'
                        },
                        {
                            // Use localized labels
                            label: getLangString('playertime.web.chart.mspt'),
                            data: msptHistory,
                            borderColor: '#FF5722',
                            backgroundColor: 'rgba(255, 87, 34, 0.1)',
                            borderWidth: 2,
                            pointRadius: 3,
                            pointHoverRadius: 5,
                            tension: 0.3,
                            yAxisID: 'y1'
                        }
                    ]
                },
                options: {
                    responsive: true,
                    maintainAspectRatio: false,
                    animation: {
                        duration: 500
                    },
                    plugins: {
                        legend: {
                            position: 'top',
                            labels: {
                                boxWidth: 12,
                                padding: 20,
                                font: {
                                    size: 12
                                }
                            }
                        },
                        tooltip: {
                            mode: 'index',
                            intersect: false,
                            bodySpacing: 8
                        }
                    },
                    scales: {
                        x: {
                            grid: {
                                display: false
                            },
                            ticks: {
                                maxRotation: 0,
                                autoSkip: true,
                                maxTicksLimit: 10
                            }
                        },
                        y: {
                            type: 'linear',
                            display: true,
                            position: 'left',
                            title: {
                                display: true,
                                // Use localized labels
                                text: getLangString('playertime.web.chart.tps'),
                                font: {
                                    weight: 'bold'
                                }
                            },
                            min: 0,
                            max: 20,
                            grid: {
                                color: 'rgba(0, 0, 0, 0.05)'
                            }
                        },
                        y1: {
                            type: 'linear',
                            display: true,
                            position: 'right',
                            title: {
                                display: true,
                                // Use localized labels
                                text: getLangString('playertime.web.chart.mspt'),
                                font: {
                                    weight: 'bold'
                                }
                            },
                            min: 0,
                            grid: {
                                drawOnChartArea: false
                            }
                        }
                    }
                }
            });
        }
    }

    function toggleTheme() {
        const currentTheme = document.documentElement.getAttribute('data-theme');
        const newTheme = currentTheme === 'light' ? 'dark' : 'light';
        document.documentElement.setAttribute('data-theme', newTheme);
        localStorage.setItem('theme', newTheme);
    }

    function handleRefresh() {
        if (this.classList.contains('loading')) return; // Prevent multiple clicks
        this.classList.add('loading');
        loadAllData().finally(() => { // Use finally to ensure loading class is removed
            setTimeout(() => { // Add a small delay for visual feedback
                this.classList.remove('loading');
            }, 500);
        });
    }

    async function loadAllData() {
        try {
            // Fetch data concurrently
            const [statsData, onlinePlayersData, playerCountsData, serverStatusData] = await Promise.all([
                fetchData('/api/stats'),
                fetchData('/api/online-players'),
                fetchData('/api/player-count'),
                fetchData('/api/server-status')
            ]);

            // Update UI with fetched data
            updateTable(statsData);
            updateOnlinePlayers(onlinePlayersData);
            updatePlayerCounts(playerCountsData);
            updateServerStatus(serverStatusData);

        } catch (error) {
            console.error('加载数据出错:', error);
            showError(getLangString('playertime.web.error.load_failed')); // Use localized error message
        }
    }

    async function fetchData(url) {
        const response = await fetch(url);
        if (!response.ok) {
            // Attempt to read error body if available
            const errorBody = await response.text().catch(() => 'Unknown Error');
            throw new Error(`HTTP error! status: ${response.status}, URL: ${url}, Body: ${errorBody}`);
        }
        return response.json();
    }

    function updateTable(data) {
        if (!elements.statsTableBody || !elements.statsTableHeader) return;

        elements.statsTableBody.innerHTML = ''; // Clear existing rows

        // Update table headers using localization
        const headers = elements.statsTableHeader.querySelectorAll('th[data-lang-key]');
        headers.forEach(th => {
            const key = th.getAttribute('data-lang-key');
            if (lang[key]) {
                th.textContent = lang[key];
            }
        });


        const sortedPlayers = Object.entries(data)
            .map(([name, statString]) => {
                // Parse the localized stat string
                const stats = {};
                // Split by " | " first
                statString.split(" | ").forEach(part => {
                    // Find the first colon to split label and value
                    const firstColonIndex = part.indexOf(':');
                    if (firstColonIndex > 0) {
                        const label = part.substring(0, firstColonIndex).trim();
                        const value = part.substring(firstColonIndex + 1).trim();
                        stats[label] = value;
                    } else {
                        // Handle cases without colon if necessary, or log a warning
                        console.warn(`Could not parse stat part: ${part}`);
                    }
                });

                // Map localized labels back to internal keys for sorting
                const internalStats = {};
                // This mapping assumes the order in the format string is consistent
                const formatParts = getLangString('playertime.stats.format').split(" | ").map(p => p.split(":")[0].trim());

                if (formatParts.length >= 3) {
                    internalStats.totalTime = stats[formatParts[0]];
                    internalStats['30Days'] = stats[formatParts[1]];
                    internalStats['7Days'] = stats[formatParts[2]];
                } else {
                    console.error("Unexpected format string structure:", getLangString('playertime.stats.format'));
                    // Fallback to using the raw statString if parsing fails
                    internalStats.totalTime = statString.split(" | ")[0]?.split(": ")[1] || "0h 00m";
                    internalStats['30Days'] = statString.split(" | ")[1]?.split(": ")[1] || "0h 00m";
                    internalStats['7Days'] = statString.split(" | ")[2]?.split(": ")[1] || "0h 00m";
                }


                return { name, stats: internalStats };
            })
            .sort((a, b) => parseTimeToSeconds(b.stats.totalTime) - parseTimeToSeconds(a.stats.totalTime));

        if (sortedPlayers.length === 0) {
            const row = elements.statsTableBody.insertRow();
            const cell = row.insertCell(0);
            cell.colSpan = 4; // Span across all columns
            cell.textContent = getLangString('playertime.web.stats_table.empty'); // Use localized empty message
            cell.style.textAlign = 'center';
            cell.style.fontStyle = 'italic';
            return;
        }


        sortedPlayers.forEach(player => {
            const row = elements.statsTableBody.insertRow();
            row.insertCell(0).textContent = player.name;
            row.insertCell(1).textContent = player.stats.totalTime;
            row.insertCell(2).textContent = player.stats['30Days'];
            row.insertCell(3).textContent = player.stats['7Days'];
        });
    }

    function updateOnlinePlayers(data) {
        if (!elements.whitelistPlayers || !elements.nonWhitelistPlayers) return;

        const updateList = (element, players, emptyMessageKey) => {
            element.innerHTML = '';
            if (players?.length > 0) {
                players.forEach(player => {
                    const li = document.createElement('li');
                    li.innerHTML = `<span class="player-name">${player.name}</span>`;
                    element.appendChild(li);
                });
            } else {
                // Use localized empty message
                element.innerHTML = `<li>${getLangString(emptyMessageKey)}</li>`;
            }
        };

        updateList(elements.whitelistPlayers, data.whitelisted, 'playertime.web.status.online_list.empty_players');
        updateList(elements.nonWhitelistPlayers, data.non_whitelisted, 'playertime.web.status.online_list.empty_non_players');
    }

    function updatePlayerCounts(data) {
        if (elements.totalCount) elements.totalCount.textContent = data.total || 0;
        if (elements.whitelistCount) elements.whitelistCount.textContent = data.whitelisted || 0;
        if (elements.nonWhitelistCount) elements.nonWhitelistCount.textContent = data.non_whitelisted || 0;
    }

    function updateServerStatus(data) {
        // Update memory info
        if (data.memory) {
            const usedMB = Math.round(data.memory.used / (1024 * 1024));
            const freeMB = Math.round((data.memory.max - data.memory.used) / (1024 * 1024));
            const percent = Math.round(data.memory.usage_percentage);

            if (memoryChart) {
                memoryChart.data.datasets[0].data = [usedMB, freeMB];
                // Update chart labels if they weren't set during initCharts (e.g., if lang loaded later)
                memoryChart.data.labels = [getLangString('playertime.web.chart.memory_used'), getLangString('playertime.web.chart.memory_free')];
                memoryChart.update();
            }

            if (elements.memoryUsed) elements.memoryUsed.textContent = usedMB;
            if (elements.memoryFree) elements.memoryFree.textContent = freeMB;
            if (elements.memoryPercent) elements.memoryPercent.textContent = percent;
        }

        // Update performance info
        if (data.server) {
            // Calculate TPS (limit between 0-20)
            const tps = Math.min(20, 1000 / (data.server.average_tick_time_ms || 50)).toFixed(1);
            const mspt = data.server.average_tick_time_ms.toFixed(1);

            // Update display
            if (elements.tpsValue) elements.tpsValue.textContent = tps;
            if (elements.msptValue) elements.msptValue.textContent = mspt;

            // Update chart data
            if (performanceChart) {
                // Update history data
                tpsHistory.shift();
                tpsHistory.push(parseFloat(tps));
                msptHistory.shift();
                msptHistory.push(parseFloat(mspt));

                // Update chart labels if they weren't set during initCharts
                performanceChart.data.datasets[0].label = getLangString('playertime.web.chart.tps');
                performanceChart.data.datasets[1].label = getLangString('playertime.web.chart.mspt');
                if (performanceChart.options.scales.y.title) performanceChart.options.scales.y.title.text = getLangString('playertime.web.chart.tps');
                if (performanceChart.options.scales.y1.title) performanceChart.options.scales.y1.title.text = getLangString('playertime.web.chart.mspt');


                // Only update data (do not recreate chart)
                performanceChart.data.datasets[0].data = tpsHistory;
                performanceChart.data.datasets[1].data = msptHistory;

                // Smooth update
                performanceChart.update('none');

                // Set color based on value
                setMetricColor(elements.tpsValue, tps, 15, 10, false); // TPS: higher is better
                setMetricColor(elements.msptValue, mspt, 50, 100, true); // MSPT: lower is better
            }

            // if (elements.avgTick) elements.avgTick.textContent = data.server.average_tick_time_ms?.toFixed(2) || '0'; // This element seems unused

        }

        if (elements.uptime) elements.uptime.textContent = data.uptime_formatted || '0'; // uptime_formatted is already localized by server
    }

    function parseTimeToSeconds(timeStr) {
        if (!timeStr) return 0;
        // Ensure parsing works even if labels are present, by only looking for h and m
        const parts = timeStr.match(/(\d+h)?\s*(\d+m)?/);
        let seconds = 0;
        if (parts) {
            if (parts[1]) { // hours part
                seconds += parseInt(parts[1].replace('h', '')) * 3600;
            }
            if (parts[2]) { // minutes part
                seconds += parseInt(parts[2].replace('m', '')) * 60;
            }
        }
        return seconds;
    }


    function showError(message) {
        const errorEl = document.createElement('div');
        errorEl.className = 'error-message';
        errorEl.textContent = message;
        document.body.appendChild(errorEl);

        setTimeout(() => {
            errorEl.classList.add('show');
            setTimeout(() => {
                errorEl.classList.remove('show');
                setTimeout(() => document.body.removeChild(errorEl), 300);
            }, 5000); // Display for 5 seconds
        }, 10); // Small delay to allow CSS transition
    }

    // Helper function to set color based on metric value and thresholds
    function setMetricColor(element, value, goodThreshold, badThreshold, reverse = false) {
        if (!element) return;

        const numValue = parseFloat(value);
        let color = '';

        if (reverse) { // For MSPT: lower is better
            if (numValue <= goodThreshold) {
                color = '#4CAF50'; // Green
            } else if (numValue <= badThreshold) {
                color = '#FFC107'; // Yellow
            } else {
                color = '#FF5722'; // Red
            }
        } else { // For TPS: higher is better
            if (numValue >= goodThreshold) {
                color = '#4CAF50'; // Green
            } else if (numValue >= badThreshold) {
                color = '#FFC107'; // Yellow
            } else {
                color = '#FF5722'; // Red
            }
        }
        element.style.color = color;
    }

});
